@@ -1,7 +1,8 @@
 # Hey emacs! This is a -*- Perl -*- script!
 # Lintian::Util -- Perl utility functions for lintian
 
-# Copyright (C) 1998 Christian Schwarz
+# Copyright © 1998 Christian Schwarz
+# Copyright © 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,8 +79,10 @@ BEGIN {
           locate_helper_tool
           drain_pipe
           signal_number2name
+          drop_relative_prefix
           dequote_name
           pipe_tee
+          read_md5sums
           untaint
           $PKGNAME_REGEX
           $PKGREPACK_REGEX
@@ -93,13 +96,21 @@ use Encode ();
 use FileHandle;
 use Scalar::Util qw(openhandle);
 
-use Lintian::Command qw(spawn);
 use Lintian::Relation::Version qw(versions_equal versions_comparator);
 
 use constant EMPTY => q{};
 use constant SPACE => q{ };
+use constant COLON => q{:};
+use constant BACKSLASH => q{\\};
 use constant NEWLINE => qq{\n};
 
+# read up to 40kB at a time.  this happens to be 4096 "tar records"
+# (with a block-size of 512 and a block factor of 20, which appear to
+# be the defaults).  when we do full reads and writes of READ_SIZE (the
+# OS willing), the receiving end will never be with an incomplete
+# record.
+use constant READ_SIZE => 4096 * 20 * 512;
+
 =head1 NAME
 
 Lintian::Util - Lintian utility functions
@@ -136,11 +147,12 @@ our $PKGNAME_REGEX = qr/[a-z0-9][-+\.a-z0-9]+/o;
 =item $PKGREPACK_REGEX
 
 Regular expression that matches "repacked" package names.  The expression is
-not anchored and does not enforce any "boundary" characters.
+not anchored and does not enforce any "boundary" characters. It should only
+be applied to the upstream portion (see #931846).
 
 =cut
 
-our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)[^-]*\-[^-]+$/o;
+our $PKGREPACK_REGEX = qr/(dfsg|debian|ds|repack)/o;
 
 =item $PKGVERSION_REGEX
 
@@ -184,40 +196,99 @@ L</parse_dpkg_control> do.  It can also emit:
 
 =cut
 
-{
+sub get_deb_info {
+    my ($path) = @_;
 
-    my $loaded = 0;
+    # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
 
-    sub get_deb_info {
-        my ($file) = @_;
+    my $loop = IO::Async::Loop->new;
 
-        # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
-        my $opts = {
-            fail => 'exception',
-            pipe_out => FileHandle->new
-        };
+    # get control tarball from deb
+    my $dpkgerror;
+    my $dpkgfuture = $loop->new_future;
+    my @dpkgcommand = ('dpkg-deb', '--ctrl-tarfile', $path);
+    my $dpkgprocess = IO::Async::Process->new(
+        command => [@dpkgcommand],
+        stdout => { via => 'pipe_read' },
+        stderr => { into => \$dpkgerror },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
 
-        if (not $loaded) {
-            $loaded++;
-            require Lintian::Command;
-        }
+            if ($status) {
+                my $message= "Non-zero status $status from @dpkgcommand";
+                $message .= COLON . NEWLINE . $dpkgerror
+                  if length $dpkgerror;
+                $dpkgfuture->fail($message);
+                return;
+            }
 
-        Lintian::Command::spawn(
-            $opts, ['dpkg-deb', '--ctrl-tarfile', $file],
-            '|', ['tar', '--wildcards', '-xO', '-f', '-', '*control']);
-        my @data = parse_dpkg_control($opts->{pipe_out});
+            $dpkgfuture->done("Done with @dpkgcommand");
+            return;
+        });
 
-        # Consume all data before exiting so that we don't kill child processes
-        # with SIGPIPE.  This will normally only be an issue with malformed
-        # control files.
-        drain_pipe($opts->{pipe_out});
-        close($opts->{pipe_out});
-        $opts->{harness}->finish;
-        return $data[0];
-    }
+    my $control;
+
+    # get the control file
+    my $tarerror;
+    my $tarfuture = $loop->new_future;
+    my @tarcommand = ('tar', '--wildcards', '-xO', '-f', '-', '*control');
+    my $tarprocess = IO::Async::Process->new(
+        command => [@tarcommand],
+        stdin => { via => 'pipe_write' },
+        stdout => { into => \$control },
+        stderr => { into => \$tarerror },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
+
+            if ($status) {
+                my $message = "Non-zero status $status from @tarcommand";
+                $message .= COLON . NEWLINE . $tarerror
+                  if length $tarerror;
+                $tarfuture->fail($message);
+                return;
+            }
+
+            $tarfuture->done("Done with @tarcommand");
+            return;
+        });
+
+    $tarprocess->stdin->configure(write_len => READ_SIZE);
+
+    $dpkgprocess->stdout->configure(
+        read_len => READ_SIZE,
+        on_read => sub {
+            my ($stream, $buffref, $eof) = @_;
+
+            if (length $$buffref) {
+                $tarprocess->stdin->write($$buffref);
+                $$buffref = EMPTY;
+            }
+
+            if ($eof) {
+                $tarprocess->stdin->close_when_empty;
+            }
+
+            return 0;
+        },
+    );
+
+    $loop->add($dpkgprocess);
+    $loop->add($tarprocess);
+
+    # awaits, and dies on failure with message from failed constituent
+    my $composite = Future->needs_all($dpkgfuture, $tarfuture);
+    $composite->get;
+
+    open(my $fh, '<', \$control);
+    my @data = parse_dpkg_control($fh);
+    close $fh;
+
+    return $data[0];
 }
 
-=item get_dsc_control (DSCFILE)
+=item get_dsc_info (DSCFILE)
 
 Convenience function for reading dsc files.  It will read the DSCFILE
 using L</read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
@@ -588,12 +659,13 @@ sub safe_qx {
 
     my $loop = IO::Async::Loop->new;
     my $future = $loop->new_future;
+    my $status;
 
     $loop->run_child(
         command => [@command],
         on_finish => sub {
             my ($pid, $exitcode, $stdout, $stderr) = @_;
-            my $status = ($exitcode >> 8);
+            $status = ($exitcode >> 8);
 
             if ($status) {
                 my $message = "Command @command exited with status $status";
@@ -605,6 +677,15 @@ sub safe_qx {
             $future->done($stdout);
         });
 
+    $loop->await($future);
+
+    if ($future->is_failed) {
+        $? = $status;
+        return $future->failure;
+    }
+
+    $? = 0;
+
     # will raise an exception in case of failure
     return $future->get;
 }
@@ -919,6 +1000,20 @@ sub check_path {
     return 0;
 }
 
+=item drop_relative_prefix(STRING)
+
+Remove an initial ./ from STRING, if present
+
+=cut
+
+sub drop_relative_prefix {
+    my ($name) = @_;
+
+    $name =~ s,^\.?/,,;
+
+    return $name;
+}
+
 =item dequote_name(STR, REMOVESLASH)
 
 Strip an extra layer quoting in index file names and optionally
@@ -930,6 +1025,8 @@ Remove initial ./ by default
 
 sub dequote_name {
     my ($name, $slsd) = @_;
+    return
+      unless defined $name;
     $slsd = 1 unless defined $slsd;
     $name =~ s,^\.?/,, if $slsd;
     # Optimise for the case where the filename does not contain
@@ -1056,7 +1153,7 @@ sub normalize_pkg_path {
     #
     # Note that @normalised will be empty if we end in the root
     # (e.g. '/' + 'usr' + '..' -> '/'), this is fine.
-    while (my $target = shift(@queue)) {
+    while (defined(my $target = shift(@queue))) {
         if ($target eq '..') {
             # are we out of bounds?
             return unless @normalised;
@@ -1195,6 +1292,93 @@ sub pipe_tee {
     return 1;
 }
 
+=item read_md5sums
+
+Untaint VALUE
+
+=cut
+
+sub read_md5sums {
+    my ($text) = @_;
+
+    # make a local copy
+    my $buffer = $text;
+
+    my %checksums;
+    my @errors;
+
+  TRY_AGAIN:
+
+    # start with checksum; processing style inspired by IO::Async::Stream
+    while ($buffer =~ s/^((?:\\)?\S{32}) [ *]//) {
+
+        my $checksum = $1;
+        my $path = EMPTY;
+        my $problematic = 0;
+
+        # leading slash in checksum indicates an escaped name
+        $problematic = 1
+          if $checksum =~ s{^\\}{};
+
+        # read up until the next newline
+        while ($buffer =~ s/^([^\n]+)\n//) {
+            my $section = $1;
+
+            # done if newline means end of record
+            unless ($problematic) {
+                $path .= $section;
+                last;
+            }
+
+            # split into individual characters
+            my @array = split(//, $section);
+
+# https://www.gnu.org/software/coreutils/manual/html_node/md5sum-invocation.html
+            my $escaped = 0;
+            for my $char (@array) {
+
+                # take next character verbatim
+                if ($escaped || $char ne BACKSLASH) {
+                    $path .= $char;
+                    $escaped = 0;
+                    next;
+                }
+
+                # start escape sequence
+                if ($char eq BACKSLASH) {
+                    $escaped = 1;
+                    next;
+                }
+            }
+
+            # do not stop inside an escape sequence
+            last
+              unless $escaped;
+
+            # add the newline that was stripped and read more
+            $path .= NEWLINE;
+        }
+
+        push(@errors, "Empty name for checksum $checksum")
+          unless length $path;
+
+        $checksums{$path} = $checksum;
+    }
+
+    if ($buffer =~ s/([^\n]*)(?:\n|\z)//) {
+        my $unreadable = $1;
+
+        # this skips empty lines
+        push(@errors, "Odd text: $unreadable")
+          if length $unreadable;
+
+        goto TRY_AGAIN
+          if length $buffer;
+    }
+
+    return (\%checksums, \@errors);
+}
+
 =item untaint(VALUE)
 
 Untaint VALUE
